@@ -1,6 +1,15 @@
-"""implements standard module 'thread' with greenlets"""
+"""Implementation of the standard :mod:`thread` module that spawns greenlets.
+
+.. note::
+    
+    This module is a helper for :mod:`gevent.monkey` and is not intended to be
+    used directly. For spawning greenlets in your applications, prefer
+    :class:`Greenlet` class.
+"""
+
 __thread = __import__('thread')
-from gevent.hub import getcurrent, GreenletExit, spawn_raw
+from gevent.hub import getcurrent, GreenletExit
+from gevent.greenlet import Greenlet
 from gevent.coros import Semaphore as LockType
 
 def get_ident(gr=None):
@@ -10,7 +19,7 @@ def get_ident(gr=None):
         return id(gr)
 
 def start_new_thread(function, args=(), kwargs={}):
-    greenlet = spawn_raw(function, *args, **kwargs)
+    greenlet = Greenlet.spawn(function, *args, **kwargs)
     return get_ident(greenlet)
 
 def allocate_lock():
